﻿-- vim: ts=2 sw=2 ai et enc=utf8 fenc=utf8
local ADDONNAME, RC = ...

if not RaidChecklist then return end

local L = RaidChecklistStrings
local NAME = L["Raid Checklist"]
local CfgHandler = {}

function CfgHandler:GetVal (info)
    local varname = info[#info]
    return RaidChecklist.db.profile[varname]
end

function CfgHandler:SetVal (info, v)
    local varname = info[#info]
    RaidChecklist.db.profile[varname] = v
    if RaidChecklist.OnConfigChange then
      RaidChecklist:OnConfigChange ()
    end
end

local config = {
  type = 'group',
  handler = CfgHandler,
  get = "GetVal", set = "SetVal",
  args = {
    about = {
      type = 'group',
      name = L.ABOUT,
      args = {
        ver = { type = 'description', name = GetAddOnMetadata("RaidChecklist", "Version"), order = 10 },
        hdr = { type = 'header', name = "Proudly brought to you by", order = 20 },
        by = { type = 'description', name = "\nAnyia of HordeYakka (Jubei'Thos) - Design, coding and partial research\n\nMalazee - Main research and ability checking\n\n\n", order = 30 },
        hdr2 = { type = 'header', name = "With special thanks to", order = 40 },
        thanks = { type = 'description', name = "\nThreepwoood - French translations\n\nMorrtin - German translations\n\n순듸windrunner(KR) - Korean translations\n\nGreenSmile - Russian translations", order = 50 },
      },
    },
    general = {
      type = 'group',
      name = L.CONFIG,
      args = {
        anchor_locked = {
          type = 'toggle', 
          name = L.ANCHORLOCKED,
          desc = L.ANCHORLOCKEDDESC,
          order = 50
        },
        show_anchor = {
          type = 'toggle',
          name = L.SHOWANCHOR,
          desc = L.SHOWANCHORDESC,
          order = 60
        },
        open_direction = {
          type = 'select',
          name = L.OPENDIRECTION,
          desc = L.OPENDIRECTIONDESC,
          values = { DOWNRIGHT = L.DOWNRIGHT, DOWNLEFT = L.DOWNLEFT, UPRIGHT = L.UPRIGHT, UPLEFT = L.UPLEFT },
          order = 100
        },
        font_size = {
          type = 'select',
          name = L.FONTSIZE,
          desc = L.FONTSIZEDESC,
          values = { normal = L.NORMAL, small = L.SMALL },
          order = 120
        },
        details_side = {
          type = 'select',
          name = L.DETAILSSIDE,
          desc = L.DETAILSSIDEDESC,
          values = { RIGHT = L.RIGHT, LEFT = L.LEFT },
          order = 130
        },
        spacer = { type = 'description', name = "", order = 150 },
        reset_anchor = {
          type = 'execute',
          name = L.RESETANCHOR,
          desc = L.RESETANCHORDESC,
          func = function ()
            RaidChecklist.button:ClearAllPoints ()
            RaidChecklist.button:SetPoint ("CENTER", 0, 0)
            RaidChecklist.db.profile.xoffs = 0
            RaidChecklist.db.profile.yoffs = 0
          end,
          order = 300
        },
        hunterpet_mode = {
          type = 'toggle',
          name = L.HUNTERPETMODE,
          desc = L.HUNTERPETMODEDESC,
          order = 310
        },
        opacity = {
          type = 'range',
          min = 0.0,
          max = 1.0,
          step = 0.05,
          name = L.OPACITY,
          desc = L.OPACITYDESC,
          order = 400
        },
        hdr2 = { type = 'description', name = L.FILTERGROUPS, order = 401 },
        group1 = { type = 'toggle', name = L.GROUP.." 1", order = 411 },
        group2 = { type = 'toggle', name = L.GROUP.." 2", order = 412 },
        group3 = { type = 'toggle', name = L.GROUP.." 3", order = 413 },
        group4 = { type = 'toggle', name = L.GROUP.." 4", order = 414 },
        group5 = { type = 'toggle', name = L.GROUP.." 5", order = 415 },
        group6 = { type = 'toggle', name = L.GROUP.." 6", order = 416 },
        group7 = { type = 'toggle', name = L.GROUP.." 7", order = 417 },
        group8 = { type = 'toggle', name = L.GROUP.." 8", order = 418 },
      }
    }
  }
}

RaidChecklist.dbdefaults = {
  profile = {
    show_anchor = true,
    anchor_locked = false,
    open_direction = "DOWNRIGHT",
    font_size = "normal",
    details_side = "RIGHT",
    anchorpoint = "CENTER",
    relpoint = "CENTER",
    x = 0,
    y = 0,
    hunterpet_mode = false,
    opacity = 0.75,
    group1 = true,
    group2 = true,
    group3 = true,
    group4 = true,
    group5 = true,
    group6 = false,
    group7 = false,
    group8 = false,
  }
}


RC.button.ADDON_LOADED = function (self, name)
  if name == ADDONNAME then
    RC.db = LibStub ("AceDB-3.0"):New ("RaidChecklistDB2", RaidChecklist.dbdefaults, true)
    LibStub ("AceConfigRegistry-3.0"):RegisterOptionsTable (NAME, config)
    config.args.profiles = LibStub ("AceDBOptions-3.0"):GetOptionsTable (RaidChecklist.db)

    RC.cfgdlg = LibStub ("AceConfigDialog-3.0")
    RC.cfgdlg:AddToBlizOptions (NAME, nil, nil, "about")
    RC.settings_panel = RC.cfgdlg:AddToBlizOptions (NAME, L.SETTINGS, NAME, "general")
    RC.cfgdlg:AddToBlizOptions (NAME, L.PROFILES, NAME, "profiles")

    if RaidChecklistDB then -- old-style config, migrate it
      for k,v in pairs (RaidChecklistDB.profile) do RC.db.profile[k] = v end
      RaidChecklistDB = nil
    end

    RC.db.RegisterCallback (RC, "OnProfileChanged", "OnConfigChange")
    RC.db.RegisterCallback (RC, "OnProfileCopied", "OnConfigChange")
    RC.db.RegisterCallback (RC, "OnProfileReset", "OnConfigChange")

    RC:OnConfigChange ()
    self:UnregisterEvent ("ADDON_LOADED")
  end
end
RC.button:RegisterEvent ("ADDON_LOADED")


