-- vim: ts=2 sw=2 ai et enc=utf8 fenc=utf8

local L = RaidChecklistStrings
local P = RaidChecklistPetFamilies

RaidChecklistData = {}
local data = RaidChecklistData

data.buffs = {
  "CRIT", "ATTACKPWR", "SPELLPWR6", "SPELLPWR10",
  "PHYSHASTE", "SPELLHASTE",
  "STATS", "ALLDMG", "AGISTR", "STAM", "ARMOR",
  "MANA", "MANAREGEN", "REPLENISHMENT", "BURSTMANA",
  "BLOODLUST",
  "PUSHBACKRES", "FIREFROSTRES", "NATURERES", "SHADOWRES",
}

data.debuffs = {
  "SPELLCRIT",
  "PHYSDMGTAKEN", "SPELLDMGTAKEN", "BLEEDDMGTAKEN",
  "ATTACKSPD", "CASTSPD", "DEMOROAR",
  "SUNDER", "MORTALSTRIKE",
}

data.dispels = {
  "OFFMAGIC", "DEFMAGIC",
  "DISEASE", "POISON",
  "CURSE", "ENRAGE"
}

data.ccs = {
  "CCBEASTS", "CCDEMONS", "CCDRAGONKIN", "CCELEMENTALS", "CCGIANTS", "CCHUMANOIDS", "CCUNDEAD", "CCANY",
  "FEARBEASTS", "FEARUNDEAD", "FEARANY"
}

data.miscs = {
  "KNOCKBACK", "INTERRUPT", "INTERRUPT2", "SNARE", "AOESNARE", "ROOT", "DISARM"
}

-- While it is possible to auto-generate this list, the sorting is much easier to handle manually
data.hunterpetabilities = {
  "CRIT", "AGISTR", "STATS", "STAM", "BLOODLUST",
  "PHYSDMGTAKEN", "SPELLDMGTAKEN", "BLEEDDMGTAKEN", "ATTACKSPD", "CASTSPD", "DEMOROAR", "SUNDER", "MORTALSTRIKE",
  "INTERRUPT", "INTERRUPT2", "DISARM"
}

-- Abuse the metatable functionality in order to make sure any abilities referenced by the providers actually exists.
local ability_typo_checker = { __index = function (table, key)
  for i,b in ipairs (table) do
    if b == key then return key end
  end
  DEFAULT_CHAT_FRAME:AddMessage ("|cffff0000>> RaidChecklist: unknown ability name: |r"..(key or "nil"))
end
}
setmetatable (data.buffs, ability_typo_checker)
setmetatable (data.debuffs, ability_typo_checker)
setmetatable (data.dispels, ability_typo_checker)
setmetatable (data.ccs, ability_typo_checker)
setmetatable (data.miscs, ability_typo_checker)

-- Convenience function to create a "provider" table
data.create_provider_table = function (self)
  local t = {}
  for i,b in ipairs (self.buffs)   do t[b] = {} end
  for i,b in ipairs (self.debuffs) do t[b] = {} end
  for i,b in ipairs (self.dispels) do t[b] = {} end
  for i,b in ipairs (self.ccs)     do t[b] = {} end
  for i,b in ipairs (self.miscs)   do t[b] = {} end
  return t
end


local B = data.buffs
local D = data.debuffs
local S = data.dispels
local C = data.ccs
local M = data.miscs

local R = { BLOODELF = "BloodElf" } -- "Filename" from UnitRace(), locale-independent

local SPEC = " "
local NOTE = "*" -- Indicates a talent which is accessible regardless of talent spec

-- Legend:
--   a - Ability
--   c - Class (localized)
--   s - Spec name (localized)
--   p - Pet family (localized)
--   l - Level requirement
--   t - Talent ID
--   n - Talent name, automatically looked up from 't'
--   i - Spell/ability ID
--   r - Rogue poison ID
--   e - Race-specific ability
--   display - Text to display for this entry
data.table = {
  { a = S.CURSE,          c = L.DRUID,        s = L.BALANCE,                          l = 24,                i = 2782                                                                   },
  { a = S.CURSE,          c = L.DRUID,        s = L.RESTORATION,                      l = 24,                i = 2782                                                                   },
  { a = S.CURSE,          c = L.MAGE,                                                 l = 30,                i = 475                                                                    },
  { a = S.CURSE,          c = L.SHAMAN,                                               l = 18,                i = 51886                                                                  },

  { a = S.DISEASE,        c = L.PALADIN,                                              l = 34,                i = 4987                                                                   },
  { a = S.DISEASE,        c = L.PRIEST,       s = L.HOLY,                             l = 22,                i = 528                                                                    },
  { a = S.DISEASE,        c = L.PRIEST,       s = L.DISCIPLINE,                       l = 22,                i = 528                                                                    },

  { a = S.ENRAGE,         c = L.DRUID,        s = L.BALANCE,                          l = 28,                i = 2908                                                                   },
  { a = S.ENRAGE,         c = L.DRUID,        s = L.RESTORATION,                      l = 28,                i = 2908                                                                   },
  { a = S.ENRAGE,         c = L.HUNTER,                                               l = 35,                i = 19801                                                                  },
  { a = S.ENRAGE,         c = L.ROGUE,                                                l = 70,                i = 5938                                                                   },

  { a = S.DEFMAGIC,       c = L.DRUID,        s = L.RESTORATION,                                  t = 88423                                                                             },
  { a = S.DEFMAGIC,       c = L.PALADIN,      s = L.HOLY,                                         t = 53551                                                                             },
  { a = S.DEFMAGIC,       c = L.PRIEST,       s = L.HOLY,                             l = 26,                i = 527                                                                    },
  { a = S.DEFMAGIC,       c = L.PRIEST,       s = L.DISCIPLINE,                       l = 26,                i = 527                                                                    },
  { a = S.DEFMAGIC,       c = L.SHAMAN,       s = L.RESTORATION,                                  t = 77130                                                                             },

  { a = S.OFFMAGIC,       c = L.HUNTER,                                               l = 35,                i = 19801                                                                  },
  { a = S.OFFMAGIC,       c = L.PRIEST,                                               l = 26,                i = 527                                                                    },
  { a = S.OFFMAGIC,       c = L.MAGE,                                                 l = 70,                i = 30449                                                                  },
  { a = S.OFFMAGIC,       c = L.SHAMAN,                                               l = 12,                i = 370                                                                    },
  { a = S.OFFMAGIC,       c = L.WARRIOR,      s = L.PROTECTION,                                              i = 23922                                                                  },

  { a = S.POISON,         c = L.DRUID,        s = L.BALANCE,                          l = 24,                i = 2782                                                                   },
  { a = S.POISON,         c = L.DRUID,        s = L.RESTORATION,                      l = 24,                i = 2782                                                                   },
  { a = S.POISON,         c = L.PALADIN,                                              l = 18,                i = 4987                                                                   },

  { a = B.AGISTR,         c = L.DEATHKNIGHT,                                          l = 65,                i = 57330                                                                  },
  { a = B.AGISTR,         c = L.HUNTER,                           p = P.CAT,                                 i = 93435                                                                  },
  { a = B.AGISTR,         c = L.HUNTER,                           p = P.SPIRITBEAST,                         i = 93435                                                                  },
  { a = B.AGISTR,         c = L.SHAMAN,                                               l = 4,                 i = 8075                                                                   },
  { a = B.AGISTR,         c = L.WARRIOR,                                              l = 20,                i = 6673                                                                   },

  { a = B.ARMOR,          c = L.PALADIN,                                              l = 5,                 i = 465                                                                    },
  { a = B.ARMOR,          c = L.SHAMAN,                                               l = 48,                i = 8071                                                                   },

  { a = B.MANA,           c = L.MAGE,                                                 l = 58,                i = 1459                                                                   },
  { a = B.MANA,           c = L.WARLOCK,                          p = P.FELHUNTER,                           i = 54424                                                                  },

  { a = B.STAM,           c = L.HUNTER,       s = L.BEASTMASTER,  p = P.SILITHID,                            i = 90364                                                                  },
  { a = B.STAM,           c = L.PRIEST,                                               l = 14,                i = 21562                                                                  },
  { a = B.STAM,           c = L.WARLOCK,                          p = P.IMP,                                 i = 6307                                                                   },
  { a = B.STAM,           c = L.WARRIOR,                                              l = 68,                i = 469                                                                    },

  { a = B.BLOODLUST,      c = L.HUNTER,       s = L.BEASTMASTER,  p = P.COREHOUND,                           i = 90355                                                                  },
  { a = B.BLOODLUST,      c = L.MAGE,                                                 l = 85,                i = 80353                                                                  },
  { a = B.BLOODLUST,      c = L.SHAMAN,                                               l = 70,                i = 2825                                                                   }, -- and 32182 (Heroism)

  { a = B.BURSTMANA,      c = L.DRUID,        s = L.BALANCE,                          l = 28,                i = 29166                                                                  },
  { a = B.BURSTMANA,      c = L.DRUID,        s = L.RESTORATION,                      l = 28,                i = 29166                                                                  },
  { a = B.BURSTMANA,      c = L.PRIEST,                                               l = 64,                i = 64904                                                                  },
  { a = B.BURSTMANA,      c = L.SHAMAN,       s = L.RESTORATION,                                  t = 16190                                                                             },

  { a = B.ATTACKPWR,      c = L.DEATHKNIGHT,  s = L.BLOOD,                                        t = 53137                                                                             }, -- and 53138
  { a = B.ATTACKPWR,      c = L.HUNTER,       s = L.MARKSMAN,                                     t = 19506                                                                             },
  { a = B.ATTACKPWR,      c = L.PALADIN,                                              l = 56,                i = 19740                                                                  },
  { a = B.ATTACKPWR,      c = L.SHAMAN,       s = L.ENHANCEMENT,                                  t = 30802                                                                             }, -- and 30808

  { a = B.CRIT,           c = L.DRUID,        s = L.FERAL,                                        t = 17007                                                                             },
  { a = B.CRIT,           c = L.HUNTER,                           p = P.WOLF,                                i = 24604                                                                  },
  { a = B.CRIT,           c = L.HUNTER,       s = L.BEASTMASTER,  p = P.DEVILSAUR,                           i = 90309                                                                  },
  { a = B.CRIT,           c = L.ROGUE,        s = L.SUBTLETY,                                     t = 51698                                                                             }, -- and 51700 + 51701
  { a = B.CRIT,           c = L.SHAMAN,       s = L.ELEMENTAL,                                    t = 51466                                                                             }, -- and 51470
  { a = B.CRIT,           c = L.WARRIOR,      s = L.FURY,                                         t = 29801                                                                             },

  { a = B.ALLDMG,         c = L.HUNTER,       s = L.BEASTMASTER,                                  t = 34460                                                                             },
  { a = B.ALLDMG,         c = L.MAGE,         s = L.ARCANE,                                       t = 82930                                                                             },
  { a = B.ALLDMG,         c = L.PALADIN,      s = L.RETRIBUTION,                                  t = 31876                                                                             },

  { a = B.PHYSHASTE,      c = L.DEATHKNIGHT,  s = L.FROST,                                        t = 55610                                                                             },
  { a = B.PHYSHASTE,      c = L.HUNTER,       s = L.SURVIVAL,                                     t = 53290                                                                             },
  { a = B.PHYSHASTE,      c = L.SHAMAN,                                               l = 30,                i = 8512                                                                   },

  { a = B.SPELLHASTE,     c = L.DRUID,        s = L.BALANCE,                                      t = 24858                                                                             },
  { a = B.SPELLHASTE,     c = L.PRIEST,       s = L.SHADOW,                                       t = 15473                                                                             },
  { a = B.SPELLHASTE,     c = L.SHAMAN,                                               l = 44,                i = 3738                                                                   },

  { a = B.SPELLPWR10,     c = L.SHAMAN,       s = L.ELEMENTAL,                                    t = 77746                                                                             },
  { a = B.SPELLPWR10,     c = L.WARLOCK,      s = L.DEMONOLOGY,                                   t = 47236                                                                             },

  { a = B.SPELLPWR6,      c = L.MAGE,                                                 l = 58,                i = 1459                                                                   },
  { a = B.SPELLPWR6,      c = L.SHAMAN,                                               l = 12,                i = 8227                                                                   },

  { a = B.STATS,          c = L.DRUID,                                                l = 30,                i = 1126                                                                   },
  { a = B.STATS,          c = L.HUNTER,       s = L.BEASTMASTER,  p = P.SHALESPIDER,                         i = 90363                                                                  },
  { a = B.STATS,          c = L.PALADIN,                                              l = 20,                i = 20217                                                                  },

  { a = B.MANAREGEN,      c = L.PALADIN,                                              l = 56,                i = 19740                                                                  },
  { a = B.MANAREGEN,      c = L.SHAMAN,                                               l = 42,                i = 5675                                                                   },
  { a = B.MANAREGEN,      c = L.WARLOCK,                          p = P.FELHUNTER,                           i = 54424                                                                  },

  { a = B.PUSHBACKRES,    c = L.PALADIN,                                              l = 42,                i = 19746                                                                  },
  { a = B.PUSHBACKRES,    c = L.SHAMAN,                                               l = 74,                i = 87718                                                                  },

  { a = B.REPLENISHMENT,  c = L.DRUID,        s = L.RESTORATION,                                  t = 48539                                                                             }, -- and 48544 + 48545
  { a = B.REPLENISHMENT,  c = L.MAGE,         s = L.FROST,                                        t = 44561                                                                             }, -- and 86500 + 86508
  { a = B.REPLENISHMENT,  c = L.PALADIN,      s = L.RETRIBUTION,                                  t = 31876                                                                             },
  { a = B.REPLENISHMENT,  c = L.PRIEST,       s = L.SHADOW,                                       t = 34914                                                                             },
  { a = B.REPLENISHMENT,  c = L.WARLOCK,      s = L.DESTRUCTION,                                  t = 30293                                                                             }, -- and 30295

  { a = B.FIREFROSTRES,   c = L.PALADIN,                                              l = 76,                i = 19891                                                                  },
  { a = B.FIREFROSTRES,   c = L.SHAMAN,                                               l = 62,                i = 8184                                                                   },

  { a = B.NATURERES,      c = L.HUNTER,                                               l = 64,                i = 20043                                                                  },
  { a = B.NATURERES,      c = L.SHAMAN,                                               l = 62,                i = 8184                                                                   },

  { a = B.SHADOWRES,      c = L.PALADIN,                                              l = 76,                i = 19891                                                                  },
  { a = B.SHADOWRES,      c = L.PRIEST,                                               l = 52,                i = 27683                                                                  },

  { a = M.DISARM,         c = L.HUNTER,                           p = P.SCORPID,                             i = 50541                                                                  },
  { a = M.DISARM,         c = L.HUNTER,                           p = P.BIRDOFPREY,                          i = 91644                                                                  },
  { a = M.DISARM,         c = L.PRIEST,       s = L.SHADOW,                                       t = 64044                                                                             },
  { a = M.DISARM,         c = L.ROGUE,                                                l = 38,                i = 51722                                                                  },
  { a = M.DISARM,         c = L.WARRIOR,                                              l = 34,                i = 676                                                                    },

  { a = C.FEARBEASTS,     c = L.HUNTER,                                               l = 36,                i = 1513                                                                   },
  { a = C.FEARUNDEAD,     c = L.PALADIN,                                              l = 78,                i = 10326                                                                  }, -- Note: also included Demons!
  { a = C.FEARANY,        c = L.WARLOCK,                                              l = 14,                i = 5782                                                                   },

  { a = M.INTERRUPT,      c = L.DEATHKNIGHT,                                          l = 57,                i = 47528                                                                  },
  { a = M.INTERRUPT,      c = L.HUNTER,                           p = P.GORILLA,                             i = 26090                                                                  },
  { a = M.INTERRUPT,      c = L.MAGE,                                                 l = 9,                 i = 2139                                                                   },
  { a = M.INTERRUPT,      c = L.ROGUE,                                                l = 14,                i = 1766                                                                   },
  { a = M.INTERRUPT,      c = L.SHAMAN,                                               l = 16,                i = 57994                                                                  },
  { a = M.INTERRUPT,      c = L.WARLOCK,                          p = P.FELHUNTER,                           i = 19647                                                                  },
  { a = M.INTERRUPT,      c = L.WARRIOR,                                              l = 38,                i = 6552                                                                   },
  -- Interrupt + silence
  { a = M.INTERRUPT,      c = L.HUNTER,       s = L.MARKSMAN,                                     t = 34490                                                                             },
  { a = M.INTERRUPT,      c = L.MAGE,         s = L.ARCANE,                                       t = 11255                                                                             }, -- and 12598
  { a = M.INTERRUPT,      c = L.PALADIN,                                              l = 54,                i = 96231                                                                  },
  { a = M.INTERRUPT,      c = L.PALADIN,      s = L.PROTECTION,                                              i = 31935                                                                  }, -- Specialization skill
  --{ a = M.INTERRUPT,      c = L.ROGUE,                                                l = 40                                                                                            },
  --{ a = M.INTERRUPT,      c = L.WARRIOR,      s = L.PROTECTION,                                   t = 12311                                                                             }, -- and 12958

  { a = M.INTERRUPT2,     c = L.DEATHKNIGHT,                                          l = 59,                i = 47476                                                                  },
  { a = M.INTERRUPT2,     c = L.DRUID,        s = L.BALANCE,                                      t = 78675                                                                             },
  { a = M.INTERRUPT2,     c = L.DRUID,        s = L.FERAL,                            l = 22,                i = 80964                                                                  },
  { a = M.INTERRUPT2,     c = L.DRUID,        s = L.FERAL,                            l = 22,                i = 80965                                                                  },
  { a = M.INTERRUPT2,     c = L.PRIEST,       s = L.SHADOW,                                       t = 15487                                                                             },
  { a = M.INTERRUPT2,     c = L.HUNTER,                           p = P.NETHERRAY,                           i = 50479                                                                  },
  { a = M.INTERRUPT2,     c = L.HUNTER,                           p = P.MOTH,                                i = 50318                                                                  },

  { a = M.INTERRUPT2,                                                                                        i = 25046,             display = L.BLOODELF_ROGUE,                         e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        i = 28730,             display = L.BLOODELF_CASTER,                        e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        i = 50613,             display = L.BLOODELF_DEATHKNIGHT,                   e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        i = 69179,             display = L.BLOODELF_WARRIOR,                       e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        i = 80483,             display = L.BLOODELF_HUNTER,                        e = R.BLOODELF },

  { a = C.CCBEASTS,       c = L.DRUID,        s = L.BALANCE,                          l = 48,                i = 2637                                                                   },
  { a = C.CCBEASTS,       c = L.DRUID,        s = L.RESTORATION,                      l = 48,                i = 2637                                                                   },
  { a = C.CCDRAGONKIN,    c = L.DRUID,        s = L.BALANCE,                          l = 48,                i = 2637                                                                   },
  { a = C.CCDRAGONKIN,    c = L.DRUID,        s = L.RESTORATION,                      l = 48,                i = 2637                                                                   },

  { a = C.CCBEASTS,       c = L.MAGE,                                                 l = 14,                i = 118                                                                    },

  { a = C.CCDEMONS,       c = L.PALADIN,      s = L.RETRIBUTION,                                  t = 20066                                                                             },
  { a = C.CCDRAGONKIN,    c = L.PALADIN,      s = L.RETRIBUTION,                                  t = 20066                                                                             },
  { a = C.CCGIANTS,       c = L.PALADIN,      s = L.RETRIBUTION,                                  t = 20066                                                                             },
  { a = C.CCHUMANOIDS,    c = L.PALADIN,      s = L.RETRIBUTION,                                  t = 20066                                                                             },
  { a = C.CCUNDEAD,       c = L.PALADIN,      s = L.RETRIBUTION,                                  t = 20066                                                                             },

  { a = C.CCUNDEAD,       c = L.PRIEST,                                               l = 32,                i = 9484                                                                   },
  { a = C.CCHUMANOIDS,    c = L.PRIEST,                                               l = 38,                i = 605                                                                    },

  { a = C.CCBEASTS,       c = L.ROGUE,                                                l = 10,                i = 6770                                                                   },
  { a = C.CCDEMONS,       c = L.ROGUE,                                                l = 10,                i = 6770                                                                   },
  { a = C.CCDRAGONKIN,    c = L.ROGUE,                                                l = 10,                i = 6770                                                                   },
  { a = C.CCHUMANOIDS,    c = L.ROGUE,                                                l = 10,                i = 6770                                                                   },

  { a = C.CCELEMENTALS,   c = L.SHAMAN,                                               l = 64,                i = 76780                                                                  },

  { a = C.CCBEASTS,       c = L.SHAMAN,                                               l = 80,                i = 51514                                                                  },
  { a = C.CCHUMANOIDS,    c = L.SHAMAN,                                               l = 80,                i = 51514                                                                  },

  { a = C.CCDEMONS,       c = L.WARLOCK,                                              l = 32,                i = 710                                                                    },
  { a = C.CCDEMONS,       c = L.WARLOCK,                                              l = 30,                i = 1098                                                                   },
  { a = C.CCELEMENTALS,   c = L.WARLOCK,                                              l = 32,                i = 710                                                                    },

  { a = C.CCHUMANOIDS,    c = L.WARLOCK,                          p = P.SUCCUBUS,                            i = 6358                                                                   },

  { a = C.CCANY,          c = L.HUNTER,                                               l = 28,                i = 1499                                                                   },

  { a = M.ROOT,           c = L.DRUID,        s = L.BALANCE,                          l = 8,                 i = 339                                                                    },
  { a = M.ROOT,           c = L.DRUID,        s = L.RESTORATION,                      l = 8,                 i = 339                                                                    },

  { a = M.SNARE,          c = L.DEATHKNIGHT,                                          l = 58,                i = 45524                                                                  },
  { a = M.SNARE,          c = L.DRUID,        s = L.FERAL,                                        t = 48483                                                                             }, -- and 48484
  { a = M.SNARE,          c = L.HUNTER,                                               l = 12,                i = 2974                                                                   },
  --{ a = M.SNARE,          c = L.MAGE,                                                 l = 56                                                                                            }, -- Frostfire Bolt
  { a = M.SNARE,          c = L.MAGE,                                                 l = 4,                 i = 116                                                                    }, -- Frost Bolt
  --{ a = M.SNARE,          c = L.MAGE,         s = L.ARCANE,                                       t = 31589                                                                             },
  { a = M.SNARE,          c = L.PRIEST,       s = L.SHADOW,                                       t = 15407                                                                             },
  { a = M.SNARE,          c = L.ROGUE,                                                l = 20,                           r = 3408                                                        }, -- Crippling poison
  --{ a = M.SNARE,          c = L.ROGUE,                                                l = 62                                                                                            }, -- Deadly Throw
  --{ a = M.SNARE,          c = L.ROGUE,        s = L.ASSASINATION,                                 t = 51625                                                                             }, -- and 51626, Deadly Brew
  --{ a = M.SNARE,          c = L.ROGUE,        s = L.COMBAT,                                       t = 31124                                                                             }, -- and 31126, Blade Twisting
  --{ a = M.SNARE,          c = L.ROGUE,        s = L.SUBTLETY,                                     t = 51692                                                                             }, -- and 51686, Waylay
  { a = M.SNARE,          c = L.SHAMAN,                                               l = 22,                i = 8056                                                                   }, -- Frostshock!
  --{ a = M.SNARE,          c = L.SHAMAN,       s = L.ENHANCEMENT,                      l = 26                                                                                            }, -- Frostbrand Weapon
  { a = M.SNARE,          c = L.WARLOCK,      s = L.AFFLICTION,                                   t = 18223                                                                             },
  { a = M.SNARE,          c = L.WARRIOR,                                              l = 36,                i = 1715                                                                   },

  { a = M.AOESNARE,       c = L.DEATHKNIGHT,                                                      t = 55666,                        display = L.DEATHKNIGHT..SPEC..L.UNHOLY..NOTE       }, -- and 55667, Unholy talent reachable by all
  { a = M.AOESNARE,       c = L.HUNTER,                                               l = 46,                i = 13809                                                                  },
  { a = M.AOESNARE,       c = L.HUNTER,       s = L.MARKSMAN,                                     t = 35100                                                                             }, -- And 35102
  { a = M.AOESNARE,       c = L.SHAMAN,                                               l = 18,                i = 2484                                                                   },
  { a = M.AOESNARE,       c = L.WARRIOR,      s = L.FURY,                                         t = 12323                                                                             },

  { a = D.BLEEDDMGTAKEN,  c = L.DRUID,        s = L.FERAL,                                                   i = 33917                                                                  }, -- Specialization skill
  { a = D.BLEEDDMGTAKEN,  c = L.HUNTER,                           p = P.BOAR,                                i = 35290                                                                  },
  { a = D.BLEEDDMGTAKEN,  c = L.HUNTER,                           p = P.HYENA,                               i = 50271                                                                  },
  { a = D.BLEEDDMGTAKEN,  c = L.HUNTER,       s = L.BEASTMASTER,  p = P.RHINO,                               i = 57386                                                                  },
  { a = D.BLEEDDMGTAKEN,  c = L.ROGUE,        s = L.SUBTLETY,                                     t = 16511                                                                             },
  { a = D.BLEEDDMGTAKEN,  c = L.WARRIOR,      s = L.ARMS,                                         t = 29836                                                                             }, -- and 29859

  { a = D.PHYSDMGTAKEN,   c = L.DEATHKNIGHT,  s = L.FROST,                                        t = 81327                                                                             }, -- and 81328
  { a = D.PHYSDMGTAKEN,   c = L.HUNTER,                           p = P.RAVAGER,                             i = 50518                                                                  },
  { a = D.PHYSDMGTAKEN,   c = L.HUNTER,       s = L.BEASTMASTER,  p = P.WORM,                                i = 55749                                                                  },
  { a = D.PHYSDMGTAKEN,   c = L.ROGUE,        s = L.COMBAT,                                       t = 51682                                                                             }, -- and 58413
  { a = D.PHYSDMGTAKEN,   c = L.WARRIOR,      s = L.ARMS,                                         t = 29836                                                                             }, -- and 29859

  { a = D.SUNDER,         c = L.DRUID,                                                l = 24,                i = 770                                                                    },
  { a = D.SUNDER,         c = L.HUNTER,                           p = P.SERPENT,                             i = 35387                                                                  },
  { a = D.SUNDER,         c = L.HUNTER,                           p = P.RAPTOR,                              i = 50498                                                                  },
  { a = D.SUNDER,         c = L.ROGUE,                                                l = 36,                i = 8647                                                                   },
  { a = D.SUNDER,         c = L.WARRIOR,      s = L.PROTECTION,                       l = 18,                i = 7386                                                                   }, -- FIXME: Protection only?

  { a = D.ATTACKSPD,      c = L.DEATHKNIGHT,                                          l = 55,                i = 59921                                                                  },
  { a = D.ATTACKSPD,      c = L.DRUID,        s = L.FERAL,                                        t = 48483                                                                             }, -- and 48484
  { a = D.ATTACKSPD,      c = L.HUNTER,                           p = P.TALLSTRIDER,                         i = 50285                                                                  },
  { a = D.ATTACKSPD,      c = L.HUNTER,                           p = P.FOX,                                 i = 90314                                                                  },
  { a = D.ATTACKSPD,      c = L.PALADIN,                                                          t = 53695,                        display = L.PALADIN..SPEC..L.PROTECTION..NOTE       }, -- and 53696. Protection talent reachable by all
  { a = D.ATTACKSPD,      c = L.ROGUE,                                                            t = 51692,                        display = L.ROGUE..SPEC..L.SUBTLETY..NOTE           }, -- and 51696. Subtlety talent reachable by all
  { a = D.ATTACKSPD,      c = L.SHAMAN,                                               l = 5,                 i = 8042                                                                   },
  { a = D.ATTACKSPD,      c = L.WARRIOR,      s = L.PROTECTION,                       l = 9,                 i = 6343                                                                   }, -- FIXME: Protectiononly?

  { a = D.CASTSPD,        c = L.DEATHKNIGHT,                                          l = 83,                i = 73975                                                                  },
  { a = D.CASTSPD,        c = L.HUNTER,                           p = P.SPOREBAT,                            i = 50274                                                                  },
  { a = D.CASTSPD,        c = L.HUNTER,       s = L.BEASTMASTER,  p = P.COREHOUND,                           i = 58604                                                                  },
  { a = D.CASTSPD,        c = L.MAGE,         s = L.ARCANE,                                       t = 31589                                                                             },
  { a = D.CASTSPD,        c = L.ROGUE,                                                l = 24,                           r = 5760                                                        },
  { a = D.CASTSPD,        c = L.WARLOCK,                                              l = 26,                i = 1714                                                                   },

  { a = D.MORTALSTRIKE,   c = L.HUNTER,                                               l = 40,                i = 82654                                                                  },
  { a = D.MORTALSTRIKE,   c = L.HUNTER,       s = L.BEASTMASTER,  p = P.DEVILSAUR,                           i = 54680                                                                  },
  { a = D.MORTALSTRIKE,   c = L.MAGE,                                                             t = 11175,                        display = L.MAGE..SPEC..L.FROST..NOTE               }, -- and 12569 + 12571. Frost talent reachable by all
  { a = D.MORTALSTRIKE,   c = L.PRIEST,                                                           t = 15273,                        display = L.PRIEST..SPEC..L.SHADOW..NOTE            }, -- and 15312 + 15313. Shadow talent reachable by all
  { a = D.MORTALSTRIKE,   c = L.ROGUE,                                                l = 32,                           r = 13218                                                       },
  { a = D.MORTALSTRIKE,   c = L.WARLOCK,      s = L.DEMONOLOGY,   p = P.FELGUARD,                            i = 30213                                                                  },
  { a = D.MORTALSTRIKE,   c = L.WARRIOR,      s = L.ARMS,                                                    i = 12294                                                                  }, -- Specialization skill
  { a = D.MORTALSTRIKE,   c = L.WARRIOR,      s = L.FURY,                                         t = 46910                                                                             },

  { a = D.DEMOROAR,       c = L.DEATHKNIGHT,  s = L.BLOOD,                                        t = 81131                                                                             }, -- and 81132
  { a = D.DEMOROAR,       c = L.DRUID,        s = L.FERAL,                            l = 15,                i = 99,                display = L.DRUID..SPEC..L.DRUIDBEAR                },
  { a = D.DEMOROAR,       c = L.HUNTER,                           p = P.BEAR,                                i = 50256                                                                  },
  { a = D.DEMOROAR,       c = L.HUNTER,                           p = P.CARRIONBIRD,                         i = 24423                                                                  },
  { a = D.DEMOROAR,       c = L.PALADIN,      s = L.PROTECTION,                                   t = 26016                                                                             },
  { a = D.DEMOROAR,       c = L.WARLOCK,                                              l = 16,                i = 702                                                                    },
  { a = D.DEMOROAR,       c = L.WARRIOR,      s = L.PROTECTION,                       l = 52,                i = 1160                                                                   },

  { a = D.SPELLCRIT,      c = L.MAGE,         s = L.FIRE,                                         t = 11095                                                                             }, -- and 12872 + 12873
  { a = D.SPELLCRIT,      c = L.WARLOCK,                                                          t = 17793,                        display = L.WARLOCK..SPEC..L.DESTRUCTION..NOTE      }, -- and 17796 + 17801. Destruction talent reachable by all

  { a = D.SPELLDMGTAKEN,  c = L.DEATHKNIGHT,  s = L.UNHOLY,                                       t = 51099                                                                             }, -- and 51160
  { a = D.SPELLDMGTAKEN,  c = L.DRUID,        s = L.BALANCE,                                      t = 48506                                                                             },
  { a = D.SPELLDMGTAKEN,  c = L.HUNTER,                           p = P.DRAGONHAWK,                          i = 34889                                                                  },
  { a = D.SPELLDMGTAKEN,  c = L.HUNTER,                           p = P.WINDSERPENT,                         i = 24844                                                                  },
  { a = D.SPELLDMGTAKEN,  c = L.ROGUE,        s = L.ASSASSINATION,                                t = 58410                                                                             },
  { a = D.SPELLDMGTAKEN,  c = L.WARLOCK,                                              l = 52,                i = 1490                                                                   },

  { a = M.KNOCKBACK,      c = L.DRUID,        s = L.BALANCE,                                      t = 50516                                                                             },
  { a = M.KNOCKBACK,      c = L.SHAMAN,       s = L.ELEMENTAL,                                               i = 51490                                                                  }, -- Specialization skill
  { a = M.KNOCKBACK,      c = L.WARLOCK,                          p = P.SUCCUBUS,                            i = 6360                                                                   },
}

for _, entry in ipairs (data.table) do
  -- Common display format unless overridden in the table above
  if not entry.display then 
    local disp = entry.c
    if entry.s then disp = disp..SPEC..entry.s end
    if entry.p then disp = disp..L.PET..entry.p end
    entry.display = disp
  end
  -- Look up the talent names once so we don't have to do that all the time
  if entry.t then
    entry.n = GetSpellInfo (entry.t)
  end
end

