-- vim: ts=2 sw=2 ai et enc=utf8 fenc=utf8

local RC = RaidChecklist
local RCData = RaidChecklistData
local L = RaidChecklistStrings

-- Map from GUID to name
RC.provider_names = {}

-- Map from (de)buff key to list of GUIDs currently providing said de(buff)
RC.providers = RCData:create_provider_table ()

local function remove_provider (rguid)
  -- Iterate over all the (de)buffs
  for key,guidlist in pairs (RC.providers) do
    guidlist[rguid] = nil
  end
  if (RC.OnRefresh) then RC:OnRefresh () end
end

local function add_provider (buff, guid)
  RC.providers[buff][guid] = 3 -- "Provided"
  if (RC.OnRefresh) then RC:OnRefresh () end
end

-- Works out which spec is used based on talent point allocations.
-- Returns 0 if no main tree is spec'd into, else 1-3 indicating tree.
local function which_spec (n1, n2, n3)
  return (n1 > n2 and n1 > n3 and 1) or (n2 > n1 and n2 > n3 and 2) or (n3 > n1 and n3 > n2 and 3) or 0
end


local specs = {
  DRUID       = { L.BALANCE,       L.FERAL,       L.RESTORATION },
  DEATHKNIGHT = { L.BLOOD,         L.FROST,       L.UNHOLY },
  HUNTER      = { L.BEASTMASTER,   L.MARKSMAN,    L.SURVIVAL},
  MAGE        = { L.ARCANE,        L.FIRE,        L.FROST },
  PALADIN     = { L.HOLY,          L.PROTECTION,  L.RETRIBUTION },
  PRIEST      = { L.DISCIPLINE,    L.HOLY,        L.SHADOW },
  ROGUE       = { L.ASSASSINATION, L.COMBAT,      L.SUBTLETY },
  SHAMAN      = { L.ELEMENTAL,     L.ENHANCEMENT, L.RESTORATION },
  WARLOCK     = { L.AFFLICTION,    L.DEMONOLOGY,  L.DESTRUCTION },
  WARRIOR     = { L.ARMS,          L.FURY,        L.PROTECTION }
}
-- Get the spec name from the baseclass and the tree (1-3) info
local specname = function (baseclass, tree)
  if tree == 0 then return nil end
  return specs[baseclass][tree]
end


RC.OnTalentUpdate = function (self, event, guid, unit, spec, n1, n2, n3)
  -- Update the talent status
  RC.missing_talents_for = RC.gt:GetTalentMissingNames ()

--if event == "LibGroupTalents_Update" then DEFAULT_CHAT_FRAME:AddMessage ("RC: updated talents for GUID "..(guid or "nil")..", unit "..unit) end

  -- Clear any previous entries for this guid
  remove_provider (guid)

  local baseclass = select (2, UnitClassBase (unit))
  local spec = specname (baseclass, which_spec (n1, n2, n3))
  local level = UnitLevel (unit)
  local class = L[baseclass]
  local _, race = UnitRace (unit)

  for _, entry in ipairs (RCData.table) do
    if entry.c == class then
      local match = true
      if match and entry.p then match = false end -- Pets are handled elsewhere
      if match and entry.l and level < entry.l then match = false end
      if match and entry.s and spec ~= entry.s then match = false end
      if match and entry.n and not RC.gt:GUIDHasTalent (guid, entry.n) then match = false end
      if match then add_provider (entry.a, guid) end
    end
    if entry.e == race then add_provider (entry.a, guid) end
  end
end

RC.OnTalentAdd = function (self, event, guid, unit, name, realm)
  local fullname = (realm and name.."-"..realm) or name
  RC.provider_names[guid] = ClassColorString (select (2,UnitClassBase (unit)))..fullname.."|r"

  -- Update based on class/level alone to start off with. Better than waiting until in inspect range!
  RC:OnTalentUpdate ("fake", guid, unit, 0, 0, 0, 0)
  -- We don't always seem to get a UNIT_PET event, so fake one to catch any initial pets
  RC.button:UNIT_PET(unit)

  --DEFAULT_CHAT_FRAME:AddMessage ("RC: discovered provider GUID "..(guid or "nil")..", named "..fullname)
end

RC.OnTalentRemove = function (self, event, guid, name, realm)
  -- Update the talent status
  RC.missing_talents_for = RC.gt:GetTalentMissingNames ()

  remove_provider (guid)
  RC.provider_names[guid] = nil

  --DEFAULT_CHAT_FRAME:AddMessage ("RC: removed provider GUID "..(guid or "nil"))
end

RC.gt = LibStub ("LibGroupTalents-1.0")
RC.gt.RegisterCallback (RC, "LibGroupTalents_Update", "OnTalentUpdate")
RC.gt.RegisterCallback (RC, "LibGroupTalents_Add", "OnTalentAdd")
RC.gt.RegisterCallback (RC, "LibGroupTalents_Remove", "OnTalentRemove")

-- Refresh on roster updates
local RCB = RC.button
function RCB:RAID_ROSTER_UPDATE()
  RC:OnRefresh ()
end
RCB:RegisterEvent("RAID_ROSTER_UPDATE")

